@@ -32,7 +32,7 @@
 #include "main.h"
 
 /* Private functions */
-static void gok_data_set_name_accessmethod_internal (char* Name);
+static void gok_data_set_name_accessmethod_internal (const char* Name);
 static const GokComposeType gok_data_compose_type_from_string (const gchar *s);
 
 /* pointer to the first GokData setting */
@@ -425,6 +425,36 @@ gboolean gok_data_get_control_values (GokControl* pControl)
 }
 */
 
+
+static void
+gok_data_fill_setting_values (gint settingValue, gchar *settingValueString, gint *Value, gchar **ValueString)
+{
+    /* set the given value to the new value */
+    if (settingValueString != NULL)
+    {
+	if (ValueString != NULL)
+	{
+	    *ValueString = settingValueString;
+	}
+	if (Value != NULL)
+	{
+	    *Value = 0;
+	}
+    }
+    else
+    {
+	if (Value != NULL)
+	{
+	    *Value = settingValue;
+	}
+	if (ValueString != NULL)
+	{
+	    *ValueString = NULL;
+	}
+    }
+}
+
+
 /**
  * gok_data_get_setting:
  * @NameAccessMethod: Name of the access method that contains the setting.
@@ -461,44 +491,37 @@ gboolean gok_data_get_setting (gchar* NameAccessMethod, gchar* NameSetting, gint
 		NameSetting = buffer;
 	}
 	
-	/* find a value for the given access method setting in our list of settings */
-	pSetting = m_pSettingFirst;
-	while (pSetting != NULL)
+	/* first look to see if we have a commandline override for the setting */
+	if (gok_main_get_scan_override () && !strcmp (gok_data_get_name_accessmethod (), NameAccessMethod) && 
+	    !strcmp (NameSetting, "movehighlighter") && gok_action_find_action (gok_main_get_scan_override (), FALSE))
 	{
-		/* check if this setting has the same access method name and setting name as given */
-		if ((strcmp (pSetting->NameAccessMethod, NameAccessMethod) == 0) &&
-			(strcmp (pSetting->Name, NameSetting) == 0))
+		gok_data_fill_setting_values (0, gok_main_get_scan_override (), Value, ValueString);
+		return TRUE;
+	}
+	else if (gok_main_get_select_override () && 
+		 !strcmp (gok_data_get_name_accessmethod (), NameAccessMethod) &&
+		 (!strcmp (NameSetting, "select") || !strcmp (NameSetting, "outputselected"))
+		 && gok_action_find_action (gok_main_get_select_override (), FALSE))
+	{
+		gok_data_fill_setting_values (0, gok_main_get_select_override (), Value, ValueString);
+		return TRUE;
+	}
+	else
+	{
+		/* find a value for the given access method setting in our list of settings */
+		pSetting = m_pSettingFirst;
+		while (pSetting != NULL)
 		{
-			/* set the given value to the new value */
-			if (pSetting->pValueString != NULL)
+			/* check if this setting has the same access method name and setting name as given */
+			if ((strcmp (pSetting->NameAccessMethod, NameAccessMethod) == 0) &&
+			    (strcmp (pSetting->Name, NameSetting) == 0))
 			{
-				if (ValueString != NULL)
-				{
-					*ValueString = pSetting->pValueString;
-				}
-				if (Value != NULL)
-				{
-					*Value = 0;
-				}
+				gok_data_fill_setting_values (pSetting->Value, pSetting->pValueString, Value, ValueString);
+				return TRUE;
 			}
-			else
-			{
-				if (Value != NULL)
-				{
-					*Value = pSetting->Value;
-				}
-				if (ValueString != NULL)
-				{
-					*ValueString = NULL;
-				}
-			}
-			
-			return TRUE;
+			pSetting = pSetting->pSettingNext;
 		}
-
-		pSetting = pSetting->pSettingNext;
 	}
-
 	return FALSE;
 }
 
@@ -870,7 +893,7 @@ void gok_data_set_keysize_priority (gboolean bFlag)
  * This function does not write to GConf it only copies the string
  */
 static void 
-gok_data_set_name_accessmethod_internal (char* Name)
+gok_data_set_name_accessmethod_internal (const char* Name)
 {
 	if (strcmp (Name, NameAccessMethod) == 0)
 	{
@@ -888,7 +911,7 @@ gok_data_set_name_accessmethod_internal (char* Name)
  *
  * Records the new access method and stores it in GConf.
  */
-void gok_data_set_name_accessmethod (char* Name)
+void gok_data_set_name_accessmethod (const char* Name)
 {
 	if (gok_main_get_access_method_override() != NULL)
 	{
