@@ -207,7 +207,7 @@ void gok_keyboard_calculate_font_size_group (GokKeyboard* pKeyboard, gint GroupN
 gboolean gok_keyboard_update_dynamic (GokKeyboard* pKeyboard);
 void gok_keyboard_update_labels (void);
 gint gok_keyboard_get_keyboards(void);
-GokKey *gok_keyboard_find_key_at (GokKeyboard *pKeyboard, gint x, gint y);
+GokKey *gok_keyboard_find_key_at (GokKeyboard *pKeyboard, gint x, gint y, GokKey *prev);
 GokKeyboardDirection gok_keyboard_parse_direction (const gchar *string);
 gint gok_keyboard_add_predictions (GokKeyboard *pKeyboard, gchar **list);
 
