@@ -30,6 +30,7 @@
 #include "gok-log.h"
 #include "gok-word-complete.h"
 #include "main.h"
+#include "gok-gconf-keys.h"
 
 static gboolean
 gok_output_can_predict (void)
@@ -50,17 +51,25 @@ gok_output_update_predictions (GokOutput    *output,
 	gunichar letter;
 	gchar *string;
 	gchar **prediction_list = NULL;
+	gchar *part = NULL;
+	GokWordComplete *complete = gok_wordcomplete_get_default ();
 
 	if (output) {
 		switch (output->Type) 
 		{
 		case OUTPUT_KEYCODE:
 		case OUTPUT_KEYSYM:
-			prediction_list = gok_wordcomplete_process_and_predict (gok_wordcomplete_get_default (),
+			prediction_list = gok_wordcomplete_process_and_predict (complete,
 										(gunichar) data,
 										gok_data_get_num_predictions ());
+			part = gok_wordcomplete_get_word_part (complete);
+			if (gok_wordcomplete_validate_word (complete, part)) { 
+			    /* don't offer to add a word that's already in the dictionary */
+			    part = NULL;
+			}
 			gok_keyboard_set_predictions (gok_main_get_current_keyboard (),
-						      prediction_list);
+						      prediction_list, 
+						      part);
 			break;
 		case OUTPUT_KEYSTRING:
 			string = (gchar *) data;
@@ -78,14 +87,20 @@ gok_output_update_predictions (GokOutput    *output,
 					}
 
 					prediction_list = gok_wordcomplete_process_and_predict (
-						gok_wordcomplete_get_default (),
+						complete,
 						letter,
 						gok_data_get_num_predictions ());
 					string = g_utf8_next_char (string);
 					letter = g_utf8_get_char (string);
 				}
+				part = gok_wordcomplete_get_word_part (complete);
+				if (gok_wordcomplete_validate_word (complete, part)) { 
+				    /* don't offer to add a word that's already in the dictionary */
+				    part = NULL;
+				}
 				gok_keyboard_set_predictions (gok_main_get_current_keyboard (), 
-							      prediction_list);
+							      prediction_list,
+							      part);				
 			}
 			break;
 		default:
@@ -431,7 +446,18 @@ void gok_output_send_to_system (GokOutput* pOutput, gboolean bWordCompletion)
 			gok_log ("g_spawn_command_line_async(\"%s\")",
 				pOutput->Name);
 
-			g_spawn_command_line_async (pOutput->Name, NULL);
+			/* check special cases */
+			if (strcmp("default:browser",pOutput->Name) == 0) {
+				/* special: default browser */
+				gchar* browser = NULL;
+				gok_gconf_get_string (gconf_client_get_default (),
+						GOK_GCONF_DEFAULT_BROWSER, &browser);
+				g_spawn_command_line_async (browser, NULL);
+			}
+			else {
+				/* not special */
+				g_spawn_command_line_async (pOutput->Name, NULL);
+			}
 		}
 		else if (pOutput->Type == OUTPUT_INTERNAL)
 		{
