@@ -310,6 +310,11 @@ gok_pointer_clip_to_window (gint *xp, gint *yp, GdkWindow *window, gint slop)
 /**
 * gok_scanner_initialize:
 * @directory: The name of the directory to read the access method files from.
+* @accessmethod: If non-NULL, overrides access method name in gconf configuration data.
+* @selectaction: If non-NULL, overrides action associated with 'select' for 
+*                the current access method in gconf configuration data.
+* @scanaction: If non-NULL, overrides access method associated with 'select' for 
+*                the current access method in gconf configuration data.
 *
 * Reads all the access methods from the given directory
 * and gets them ready to go.
@@ -317,7 +322,8 @@ gok_pointer_clip_to_window (gint *xp, gint *yp, GdkWindow *window, gint slop)
 * returns: TRUE if the access methods were initialized, FALSE if not. Don't use the 
 * access methods if this fails.
 **/
-gboolean gok_scanner_initialize (const gchar *directory)
+gboolean gok_scanner_initialize (const gchar *directory, const gchar *accessmethod, 
+				 const gchar *selectaction, const gchar *scanaction)
 {
 	DIR* pDirectoryAccessMethods;
 	struct dirent* pEntry;
@@ -376,7 +382,22 @@ gboolean gok_scanner_initialize (const gchar *directory)
 	/* get the rates for all the access methods */
 	gok_scanner_update_rates ();
 
-	/* set the current access method */
+	if (accessmethod) /* we've specified the name in the args list */
+	{
+	    m_pAccessMethodCurrent = m_pAccessMethodFirst;
+	    while (m_pAccessMethodCurrent)
+	    { 
+		if (!strcmp (m_pAccessMethodCurrent->Name, accessmethod)) 
+		{
+		    gok_data_set_name_accessmethod (accessmethod);
+		    gok_scanner_reset_access_method ();
+		    g_message ("using access method %s", accessmethod);
+		    return TRUE;
+		}
+		m_pAccessMethodCurrent = m_pAccessMethodCurrent->pAccessMethodNext;
+	    }
+	}
+	/* set the current access method from gconf value (note fall-through if match above failed) */
 	return gok_scanner_change_method (gok_data_get_name_accessmethod());
 }
 
@@ -974,7 +995,7 @@ gboolean gok_scanner_read_description (xmlDoc* pDoc, xmlNode* pNode, GokAccessMe
 		pStringDescription = xmlNodeListGetString (pDoc, pNode->xmlChildrenNode, 1);
 		strncpy (pAccessMethod->Description, (gchar *)pStringDescription, MAX_DESCRIPTION_TEXT);
 		gok_log ("read access method description: [%s]", pStringDescription);
-		g_free (pStringDescription);
+		xmlFree (pStringDescription);
 		gok_log_leave();
 		return TRUE;
 	}
@@ -2316,7 +2337,7 @@ void gok_scanner_input_motion (gint *motion_data, gint n_axes)
 		x -= o_x;
 		y -= o_y;
 	}
-	pKey = gok_keyboard_find_key_at (pKeyboard, x, y);
+	pKey = gok_keyboard_find_key_at (pKeyboard, x, y, m_pContainingKey);
 	gok_keyboard_unpaint_pointer (pKeyboard, pWindow);
 	if (m_pContainingKey != pKey) {
 		gok_scanner_on_key_leave (m_pContainingKey);
@@ -3325,6 +3346,9 @@ gok_scanner_current_state_uses_corepointer (void)
 			retval = TRUE;
 		else
 			g_free (input_device_name);
+		
+		if (retval && (gok_main_get_inputdevice_name () != NULL))
+			retval = FALSE;
 	}
 
 	return retval;
