@@ -1275,7 +1275,6 @@ gok_spy_distinguish_node_name (AccessibleNode *node)
 	if (node->paccessible) 
 	    relations = Accessible_getRelationSet(node->paccessible);
 	if (relations != NULL) {
-#ifdef NEVER		
 	 while (relations[i]) {
 		if (!distinguished) {
 			type = AccessibleRelation_getRelationType(relations[i]);
@@ -1288,8 +1287,7 @@ gok_spy_distinguish_node_name (AccessibleNode *node)
 						s = Accessible_getName (labelAccessible);
 						if (s != NULL) {
 							if ( strlen (s) != 0 )  {
-								gchar buffer[64];
-								buffer[63] = '\0';
+								gchar buffer[64] = "";
 								if (node->pname) 
 								{
 									if (strlen (node->pname)) 
@@ -1298,6 +1296,7 @@ gok_spy_distinguish_node_name (AccessibleNode *node)
 									}
 									g_free (node->pname); 
 								}
+								buffer[63] = '\0'; /* ensure null termination */
 								node->pname = g_strconcat (s, " ", buffer, NULL);
 								SPI_freeString (s);
 								Accessible_unref (labelAccessible);
@@ -1315,7 +1314,6 @@ gok_spy_distinguish_node_name (AccessibleNode *node)
 		AccessibleRelation_unref(relations[i]);
 		i++;
 	 }
-#endif
 	 g_free (relations);
 	}
 }
@@ -2252,18 +2250,26 @@ gok_spy_search_component_list (Accessible *rootAccessible, GokSpyUIFlags keyboar
 	parent_role = Accessible_getRole (parent);
 	child_count = Accessible_getChildCount (parent);
 	/* MAX_CHILD_COUNT is a bit of a kludge and will cause certain table items to be unreachable via UI Grab */
-	for (i = 0; i < child_count && i < MAX_CHILD_COUNT; ++i)
+	if (child_count > MAX_CHILD_COUNT)
+		child_count = MAX_CHILD_COUNT;
+	for (i = 0; i < child_count; ++i)
 	{
 		AccessibleNodeFlags flags;
 		flags.value = 0;
 		child = Accessible_getChildAtIndex (parent, i);
-		gok_spy_accessible_implicit_ref (child);
 		if (child)
 		{
+			gok_spy_accessible_implicit_ref (child);
 			flags.value = 0;
 			role = Accessible_getRole (child);
 			if (keyboard_ui_flags.data.menus && gok_spy_is_menu_role (role))
 			{
+				if (parent_role == SPI_ROLE_MENU_BAR &&
+				    !gok_spy_is_visible (child))
+				{
+					gok_spy_accessible_unref (child);
+					continue;
+				}
 				ui_flags.data.menus = flags.data.is_menu = TRUE;
 			}
 			else if (keyboard_ui_flags.data.toolbars && 
