@@ -335,12 +335,33 @@ gboolean
 gok_wordcomplete_increment_word_frequency (GokWordComplete *complete, const gchar* pWord)
 {
 	GokWordCompleteClass *klass = GOK_WORDCOMPLETE_GET_CLASS (complete);
-	if (*klass->increment_word_frequency)
+	if (*klass->increment_word_frequency && pWord && g_utf8_strlen (pWord, -1))
 		return (* klass->increment_word_frequency)(complete, pWord);
 	else
 		return FALSE;
 }
 
+/**
+* gok_wordcomplete_validate_word:
+* 
+* Determines whether a word is known to the completion engine.
+* For dictionary-based engines this implies that the word is in the dictionary;
+* some other sorts of engines may return TRUE if the word matches a heuristic,
+* but may not guarantee that the word will appear in subsequent prediction
+* runs.
+*
+* returns: TRUE if the word is know to the completion engine, FALSE if not.
+**/
+gboolean 
+gok_wordcomplete_validate_word (GokWordComplete *complete, const gchar* pWord)
+{
+	GokWordCompleteClass *klass = GOK_WORDCOMPLETE_GET_CLASS (complete);
+	if (*klass->validate_word)
+		return (* klass->validate_word) (complete, pWord);
+	else
+		return FALSE;
+}
+
 /**
 * gok_wordcomplete_get_aux_dictionaries:
 * 
@@ -653,6 +674,8 @@ gok_wordcomplete_real_process_unichar (GokWordComplete *complete, gunichar lette
 		prefix = gok_wordcomplete_unichar_pop (complete);
 		break;
 	case GOK_WORDCOMPLETE_CHAR_TYPE_DELIMITER:
+		gok_wordcomplete_increment_word_frequency (complete,
+							   gok_wordcomplete_get_word_part (complete));
 		gok_wordcomplete_reset (complete);
 		break;
 	case GOK_WORDCOMPLETE_CHAR_TYPE_COMPOSE:
