@@ -116,9 +116,30 @@ static gint m_GeometryY = -1;
 /* pointer to the gok command predictor */
 static Gok_Predictor m_pCommandPredictor = NULL;
 
+/* poptOption storage structure */
+typedef struct _GokArgs {
+	char* accessmethodname;
+	char* inputdevicename;
+	char* mainkeyboardname;
+	char* custom_compose_kbd_name;
+	int display_keyboard_editor;
+	char *geometry;
+	int geometry_bitmask;
+	gboolean is_login;
+	int remember_geometry;
+	int display_settings_dialog;
+	gboolean use_extras;
+	char* scanactionname;
+	char* selectactionname;
+	gboolean list_actions;
+	gboolean list_accessmethods;
+} GokArgs;
+
+static GokArgs gok_args;
+
 /* Private functions */
 static void gok_main_read_rc (void);
-static void gok_main_initialize_access_methods (void);
+static void gok_main_initialize_access_methods (GokArgs *args);
 static void gok_main_initialize_wordcomplete (void);
 static void gok_main_initialize_commandprediction (void);
 static gboolean gok_main_display_scan_main(void);
@@ -127,19 +148,6 @@ static void gok_main_object_state_listener (Accessible* pAccessible);
 static gboolean gok_main_has_xkb_extension ();
 static gboolean gok_main_check_sticky_keys (GtkWidget *widget);
 
-/* poptOption storage */
-static char* accessmethodname = NULL;
-static char* inputdevicename = NULL;
-static char* mainkeyboardname = NULL;
-static char* custom_compose_kbd_name = NULL;
-static int bDisplayKeyboardEditor = 0;
-static char *geometry = NULL;
-static int geometry_bitmask;
-static int bLoginOption = 0;
-static int bRememberGeometryOption = 0;
-static int bDisplaySettingsDialog = 0;
-static int bExtras = 0;
-
 static gint _screen_height, _screen_width;
 
 
@@ -148,16 +156,16 @@ static struct poptOption options[] = {
 		"access-method", 
 		'a',
 		POPT_ARG_STRING, 
-		&accessmethodname, 
+		&gok_args.accessmethodname, 
 		0,
-		N_("Use the specified access method. NAME is a string and can be found in the various access method files (.xam) assigned to the \"name\" property of <gok:accessmethod> tag. Note this is not necessarily the same as the name of the .xam file."), 
+		N_("Use the specified access method. NAME is a string and can be found in the various access method files (.xam) assigned to the \"name\" property of <gok:accessmethod> tag. Note this is not necessarily the same as the name of the .xam file. (See --list-accessmethods)"), 
 		N_("NAME")
 	},
 	{
 		"editor", 
 		'e', 
 		POPT_ARG_NONE, 
-		&bDisplayKeyboardEditor, 
+		&gok_args.display_keyboard_editor, 
 		0,
 		N_("Start the GOK keyboard editor"), 
 		NULL},
@@ -165,7 +173,7 @@ static struct poptOption options[] = {
 		"extras", 
 		'\0', 
 		POPT_ARG_NONE, 
-		&bExtras, 
+		&gok_args.use_extras, 
 		0,
 		N_("Use special, but possibly unstable, gok stuff"),
 		NULL},
@@ -173,17 +181,16 @@ static struct poptOption options[] = {
 		"geometry", 
 		'\0', 
 		POPT_ARG_STRING, 
-		&geometry, 
+		&gok_args.geometry, 
 		0,
 		N_("Whenever --geometry is not used gok remembers its position between invocations and starts in the position that it had when it was last shutdown.  When --geometry is used gok positions itself within the rectangular area of screen described by the given X11 geometry specification.  When --geometry is used gok does not remember its position when it shuts down.  This behaviour can be changed with the --remembergeometry flag which forces gok to remember its position when shutdown even when it was started with --geometry."),
 		N_("GEOMETRY")
 	},
 	{
-		/* TODO: NOT USED */
 		"input-device", 
 		'i', 
 		POPT_ARG_STRING, 
-		&inputdevicename, 
+		&gok_args.inputdevicename, 
 		0,
 		N_("Use the specified input device"), 
 		N_("DEVICENAME")
@@ -192,34 +199,75 @@ static struct poptOption options[] = {
 		"keyboard", 
 		'k', 
 		POPT_ARG_STRING, 
-		&mainkeyboardname, 
+		&gok_args.mainkeyboardname, 
 		0,
 		N_("Start GOK with the specified keyboard."), 
 		N_("KEYBOARDNAME")
 	},
+	{
+		"list-accessmethods", 
+		'\0', 
+		POPT_ARG_NONE, 
+		&gok_args.list_accessmethods, 
+		0,
+		N_("List the access methods that can used as options to other arguments."),
+		NULL
+	},
+	{
+		"list-actions", 
+		'\0', 
+		POPT_ARG_NONE, 
+		&gok_args.list_actions, 
+		0,
+		N_("List the actions that can used as options to other arguments."),
+		NULL
+	},
 	{
 		"login", 
 		'l', 
 		POPT_ARG_NONE, 
-		&bLoginOption, 
+		&gok_args.is_login, 
 		0,
 		N_("GOK will be used to login"),
 		NULL
 	},
+
+
 	{
 		"remembergeometry", 
 		'\0', 
 		POPT_ARG_NONE,
-		&bRememberGeometryOption, 
+		&gok_args.remember_geometry, 
 		0,
 		N_("Can be used with --geometry.  Forces GOK to remember its position when shutdown even when it was started with --geometry.  Please see the discussion under the --geometry flag for more information."),
 		NULL
 	},
+	{
+		/* primarily for use at login since there is no gconf */
+		/* note: not enough info for 5-switch directed scanning */
+		"scan-action", 
+		'\0', 
+		POPT_ARG_STRING, 
+		&gok_args.scanactionname, 
+		0,
+		N_("Start GOK and hook this action to scan operations. (See --list-actions)"), 
+		NULL
+	},
+	{
+		/* primarily for use at login since there is no gconf */
+		"select-action", 
+		'\0', 
+		POPT_ARG_STRING, 
+		&gok_args.selectactionname, 
+		0,
+		N_("Start GOK and hook this action to select operations. (See --list-actions)"), 
+		NULL
+	},
 	{
 		"settings", 
 		's', 
 		POPT_ARG_NONE, 
-		&bDisplaySettingsDialog, 
+		&gok_args.display_settings_dialog, 
 		0,
 		N_("Open the settings dialog box when GOK starts"),
 		NULL
@@ -258,11 +306,30 @@ static int gok_sig_handler(int sig)
 	return -1;  /* this line gets rid of compiler warning */ 
 }
 
+static void
+gok_args_init (GokArgs *args)
+{
+	/* initialize command line option (popt) storage */
+	args->accessmethodname = NULL;
+	args->inputdevicename = NULL;
+	args->mainkeyboardname = NULL;
+	args->custom_compose_kbd_name = NULL;
+	args->display_keyboard_editor = FALSE;
+	args->geometry = NULL;
+	args->geometry_bitmask;
+	args->is_login = 0;
+	args->remember_geometry = FALSE;
+	args->display_settings_dialog = FALSE;
+	args->use_extras = FALSE;
+	args->scanactionname = NULL;
+	args->selectactionname = NULL;
+}
+
 /* callback for gok window state events */
 gboolean
 on_gok_window_state_event ( GtkWidget       *widget,
-							GdkEventWindowState  *event,
-							gpointer         user_data)
+			    GdkEventWindowState  *event,
+			    gpointer         user_data)
 {
 	if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED) {
 		if (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED) {
@@ -275,9 +342,61 @@ on_gok_window_state_event ( GtkWidget       *widget,
 const gchar *
 gok_main_get_custom_compose_kbd_name ()
 {
-	return custom_compose_kbd_name;
+	return gok_args.custom_compose_kbd_name;
 }
 
+static gboolean
+gok_application_set_safe (LoginHelper *helper, gboolean safe)
+{
+    return TRUE; /* TODO: actually go to safe mode here! */
+}
+
+static LoginHelperDeviceReqFlags
+gok_application_get_device_reqs (LoginHelper *helper)
+{
+    return LOGIN_HELPER_GUI_EVENTS | LOGIN_HELPER_POST_WINDOWS | LOGIN_HELPER_EXT_INPUT | 
+	LOGIN_HELPER_AUDIO_OUT;
+    /* 
+     * FIXME return values based on current access method, i.e. may include
+     * mouse/keyboard, or omit AUDIO.
+     */
+}
+
+static Window*
+gok_application_get_raise_windows (LoginHelper *helper)
+{
+    Window *mainwin = NULL;
+    GtkWidget *widget = gok_main_get_main_window ();
+
+    if (widget)
+    {
+	mainwin = g_new0 (Window, 2);
+	mainwin[0] = GDK_WINDOW_XWINDOW (widget->window);
+	mainwin[1] = None;
+    }
+    return mainwin;
+}
+
+static void
+gok_application_init (GokApplication *app)
+{
+    ;
+}
+
+static void
+gok_application_class_init (GokApplicationClass *klass)
+{
+    LoginHelperClass *login_helper_class = LOGIN_HELPER_CLASS(klass);
+
+    login_helper_class->get_raise_windows = gok_application_get_raise_windows;
+    login_helper_class->get_device_reqs = gok_application_get_device_reqs;
+    login_helper_class->set_safe = gok_application_set_safe;
+}
+
+BONOBO_TYPE_FUNC (GokApplication,
+		  LOGIN_HELPER_TYPE,
+		  gok_application)
+
 /**
 * main
 * @argc: ignored
@@ -317,6 +436,24 @@ gint main (gint argc, gchar *argv[])
 	return 0;
 }
 
+/* private helper */
+gint
+comparebasenames (gconstpointer a, gconstpointer b)
+{
+	gchar* a_file;
+	gchar* b_file;
+	gint returncode;
+	a_file = g_path_get_basename(a);
+	b_file = g_path_get_basename(b);
+
+	returncode = g_strcasecmp(a_file, b_file);
+	
+	g_free (a_file);
+	g_free (b_file);
+	
+	return returncode;
+}
+
 /**
 * gok_main_open
 * @argc: ignored
@@ -330,13 +467,16 @@ gint
 gok_main_open(gint argc, gchar *argv[])
 {
 	GnomeProgram *gok_program;
+
 	/* initialize member data */
 	m_pForegroundWindowAccessible = NULL;
 	m_pKeyboardFirst = NULL;
 	m_pWindowMain = NULL;
 	m_pCurrentKeyboard = NULL;
 	m_countIgnoreConfigure = 0;
-	
+
+	gok_args_init (&gok_args);
+
 	/* Initialize Gnome */
 	gok_program = gnome_program_init (PACKAGE, VERSION,
 		LIBGNOMEUI_MODULE, argc, argv,
@@ -345,45 +485,68 @@ gok_main_open(gint argc, gchar *argv[])
 	        GNOME_PARAM_APP_DATADIR, DATADIR"/gok", 
 		LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
 		NULL);
-
-	if (bLoginOption) {
-		GObject      *gobj;
-		BonoboObject *bobj;
-		int          ret;
-		const gchar  *display_name;
-		GSList       *reg_env = NULL;
 		
-		if (!bonobo_init (&argc, argv))
-			g_error ("Could not initialize oaf / Bonobo");
+	
+	if (!bonobo_init (&argc, argv))
+	{
+	    if (gok_args.is_login)
+		g_error ("Could not initialize Bonobo");
+	    else
+		g_warning ("Could not initialize Bonobo");
+	}
+	
+	else
+	{
+	    BonoboObject *obj;
+	    const gchar  *display_name;
+	    GSList       *reg_env = NULL;
+	    Bonobo_RegistrationResult ret;
+	    CORBA_Environment ev;
 
-		gobj = g_object_new (BONOBO_TYPE_APPLICATION, NULL);
-		bobj = BONOBO_OBJECT (gobj);
+	    bonobo_activate ();
 
-		display_name = gdk_display_get_name (gdk_display_get_default ());
-		reg_env = bonobo_activation_registration_env_set (reg_env, "DISPLAY",
-								  display_name);
+	    CORBA_exception_init (&ev);
 
-		ret = bonobo_activation_register_active_server ("OAFIID:GNOME_Gok_Factory", bonobo_object_corba_objref (bobj), reg_env);
-		bonobo_activation_registration_env_free (reg_env);
-
-		if (ret != Bonobo_ACTIVATION_REG_SUCCESS)
-			return 1;
+	    obj = BONOBO_OBJECT (g_object_new (GOK_TYPE_APPLICATION, NULL));
+	    display_name = gdk_display_get_name (gdk_display_get_default ());
+	    reg_env = bonobo_activation_registration_env_set (reg_env, "DISPLAY",
+							      display_name);
+	    
+	    ret = bonobo_activation_register_active_server ("OAFIID:GNOME_GOK:1.0", 
+							    BONOBO_OBJREF (obj), reg_env);
+	    
+	    bonobo_activation_registration_env_free (reg_env);
+	
+	    if (ret != Bonobo_ACTIVATION_REG_SUCCESS)
+	    {
+		if (gok_args.is_login)
+		    return 1;
+		else {
+		    gchar *reason = NULL;
+		    if (ret == Bonobo_ACTIVATION_REG_ALREADY_ACTIVE) reason = "already active";
+		    else if (ret == Bonobo_ACTIVATION_REG_ERROR) reason = "error";
+		    else if (ret == Bonobo_ACTIVATION_REG_NOT_LISTED) reason = "not listed";
+		    else if (ret == Bonobo_ACTIVATION_REG_SUCCESS) reason = "success";
+		    else reason = "unknown reason";
+		    g_warning ("Error registering GOK as a server: %s", reason);
+		}
+	    }
 	}
 	
 	/* Parse geometry */
-	if (geometry != NULL)
+	if (gok_args.geometry != NULL)
 	{
-		geometry_bitmask = XParseGeometry (geometry, &m_GeometryX,
+		gok_args.geometry_bitmask = XParseGeometry (gok_args.geometry, &m_GeometryX,
 		                                   &m_GeometryY,
 		                                   (unsigned int *)&m_GeometryWidth,
 		                                   (unsigned int *)&m_GeometryHeight);
 
-		if ( (geometry_bitmask & XValue)
-		     && (geometry_bitmask & YValue)
-		     && (geometry_bitmask & WidthValue)
-		     && (geometry_bitmask & HeightValue) )
+		if ( (gok_args.geometry_bitmask & XValue)
+		     && (gok_args.geometry_bitmask & YValue)
+		     && (gok_args.geometry_bitmask & WidthValue)
+		     && (gok_args.geometry_bitmask & HeightValue) )
 		{
-			gok_bounds_get_upper_left (geometry_bitmask,
+			gok_bounds_get_upper_left (gok_args.geometry_bitmask,
 							m_GeometryX, m_GeometryY,
 							m_GeometryWidth,
 							m_GeometryHeight,
@@ -412,6 +575,62 @@ gok_main_open(gint argc, gchar *argv[])
 	/* read the user profile data */
 	gok_data_initialize ();
 
+	/* check for --list-actions */
+	if (gok_args.list_actions) { 
+		GConfClient *gconf_client = gconf_client_get_default ();
+		GError* error = NULL;
+		GSList* list = gconf_client_all_dirs (gconf_client,
+			GOK_GCONF_ACTIONS, &error);
+		
+		fprintf (stderr, "\nGOK Actions:\n");
+		if (error == NULL) {
+			list = g_slist_sort (list, (GCompareFunc)comparebasenames);
+			GSList* listhead = list;
+			while (list) {
+				fprintf(stderr,"%s\n",g_path_get_basename(list->data));
+				g_free(list->data);
+				list = g_slist_next(list);
+			}
+			g_slist_free (listhead);
+		}
+		else {
+			fprintf (stderr, "mousebutton<n>\nswitch<n>\ndwell\n\n");
+			fprintf (stderr, "\n\n***WARNING: There was an error querying gconf.\nExhaustive list unavailable.\n\n");
+		}
+			
+		_exit(0);
+	}
+
+	/* check for --list-accessmethods */
+	if (gok_args.list_accessmethods) {
+		/* note: code dupe of check for --list-actions */
+		GConfClient *gconf_client = gconf_client_get_default ();
+		GError* error = NULL;
+		GSList* list = gconf_client_all_dirs (gconf_client,
+			GOK_GCONF_ACCESS_METHOD_SETTINGS, &error);
+		
+		fprintf (stderr, "\nGOK Access Methods:\n");
+		if (error == NULL) {
+			gchar* base;
+			list = g_slist_sort (list, (GCompareFunc)comparebasenames);
+			GSList* listhead = list;
+			while (list) {
+				base = g_path_get_basename(list->data);
+				fprintf(stderr,"%s\n",base);
+				g_free(base);
+				g_free(list->data);
+				list = g_slist_next(list);
+			}
+			g_slist_free (listhead);
+		}
+		else {
+			fprintf (stderr, "\nGOK Access Methods:\nautomaticscanning\ndirected\ndirectselection\ndwellselection\ninversescanniing\nkeyautoscanning\nkeyinversescanning\n\n");
+			fprintf (stderr, "\n\n***WARNING: There was an error querying gconf.\nExhaustive list unavailable.\n\n");
+		}
+			
+		_exit(0);
+	}
+	
 	/* initialize the word prediction engine with gconf values */
 	if (gok_data_get_use_aux_dictionaries ())
 		gok_wordcomplete_set_aux_dictionaries (gok_wordcomplete_get_default (),
@@ -428,7 +647,7 @@ gok_main_open(gint argc, gchar *argv[])
 	}
 
 	/* display the keyboard editor not the GOK */
-	if (bDisplayKeyboardEditor)
+	if (gok_args.display_keyboard_editor)
 	{
 		SPI_init ();
 		gok_editor_run();
@@ -466,7 +685,7 @@ gok_main_open(gint argc, gchar *argv[])
 	}
 
 	/* initialize the access methods */
-	gok_main_initialize_access_methods ();
+	gok_main_initialize_access_methods (&gok_args);
 
 	/* initialize the SPI */
 	SPI_init ();
@@ -498,14 +717,14 @@ gok_main_open(gint argc, gchar *argv[])
 	gok_keyboard_initialize();
 	gok_branchbackstack_initialize();
 
-	fprintf (stderr, "login mode = %s\n", bLoginOption ? "true" : "false");
-	if (!bLoginOption) {
+	fprintf (stderr, "login mode = %s\n", gok_args.is_login ? "true" : "false");
+	if (!gok_args.is_login) {
 		gok_main_initialize_wordcomplete ();
 		gok_main_initialize_commandprediction ();
 	}
 
 	/* create the settings dialog */
-	if (gok_settingsdialog_open (bDisplaySettingsDialog) == FALSE)
+	if (gok_settingsdialog_open (gok_args.display_settings_dialog) == FALSE)
 	{
 		gok_main_display_error (_("Can't create the settings dialog window!"));
 		return GOK_ERROR_SETTINGSDIALOG_OPEN;
@@ -514,20 +733,20 @@ gok_main_open(gint argc, gchar *argv[])
 
 	/* a specified keyboard takes precedance, even if the login flag is specifed
 	   we use the login keyboard, otherwise the regular main keyboard */
-	if (mainkeyboardname == NULL) {
-		if (bLoginOption) {
-			mainkeyboardname = (gchar*) 
+	if (gok_args.mainkeyboardname == NULL) {
+		if (gok_args.is_login) {
+			gok_args.mainkeyboardname = (gchar*) 
 				g_malloc (strlen (GOKLOGINKEYBOARDNAME) + 1);
-			strcpy (mainkeyboardname, (gchar *) GOKLOGINKEYBOARDNAME);
+			strcpy (gok_args.mainkeyboardname, (gchar *) GOKLOGINKEYBOARDNAME);
 		}
 		else {
-			mainkeyboardname = (gchar*) 
+			gok_args.mainkeyboardname = (gchar*) 
 				g_malloc (strlen (GOKMAINKEYBOARDNAME) + 1);
-			strcpy (mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
+			strcpy (gok_args.mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
 		}
 	}
 
-	if (!bLoginOption) {
+	if (!gok_args.is_login) {
 		gok_main_check_accessibility();
 		gok_main_warn_if_corepointer_mode (_("You are using GOK in \'core pointer\' mode."), FALSE);
 	}
@@ -564,7 +783,7 @@ gok_log ("finished check");
 static gboolean
 gok_main_display_scan_main()
 {
-	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
+	if (gok_main_display_scan ((GokKeyboard*)NULL, gok_args.mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 	{
 		/* if no "main" keyboard then display first keyboard in list */
 		if (gok_main_display_scan ((GokKeyboard*)NULL, NULL, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
@@ -826,23 +1045,22 @@ gok_main_object_state_listener (Accessible* pAccessible)
 void gok_main_motion_listener (gint n_axes, int *motion_data, long mods, long timestamp)
 {
         GtkWidget *window = gok_main_get_main_window ();
-	Display *display;
-	GdkWindow *root;
-	gint dx, dy;
 	if (window) 
 	{
-		display = GDK_WINDOW_XDISPLAY (window->window);
-		root = gdk_screen_get_root_window (gdk_drawable_get_screen (window->window));
+#ifdef GOK_LOG_INPUT_EVENTS
 		gok_log_enter();
 		gok_log ("%d axes: [%d] [%d] mods[%lx] timestamp[%ld]", n_axes,
 			motion_data[0], (n_axes > 1) ? motion_data[1] : 0, 
 			mods, timestamp);
 		gok_log_leave ();
+#endif
 		gok_scanner_input_motion (motion_data, n_axes);
 
-		dx = motion_data[0] * gok_data_get_valuator_sensitivity ();
-		dy = motion_data[1] * gok_data_get_valuator_sensitivity ();
 		if (gok_data_get_drive_corepointer ()) {
+			Display *display;
+			GdkWindow *root;
+			display = GDK_WINDOW_XDISPLAY (window->window);
+			root = gdk_screen_get_root_window (gdk_drawable_get_screen (window->window));
 			XWarpPointer (display, None, 
 				      GDK_WINDOW_XWINDOW (root),
 				      0, 0, 0, 0, motion_data[0], motion_data[1]);
@@ -1308,7 +1526,7 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	}
 	
 	/* layout the keys on the keyboard */
-	if (gok_keyboard_layout (pKeyboard, KEYBOARD_LAYOUT_NORMAL, 
+	if (gok_keyboard_layout (pKeyboard, pKeyboard->LayoutType, 
 		 	/*KEYBOARD_SHAPE_WIDE KEYBOARD_SHAPE_SQUARE*/ 
 			KEYBOARD_SHAPE_KEYSQUARE
 			, FALSE) == FALSE)
@@ -1347,7 +1565,6 @@ gboolean gok_main_ds (GokKeyboard* pKeyboard)
 	return TRUE;
 }
 
-
 /**
 * gok_main_create_window
 * 
@@ -1533,7 +1750,7 @@ gok_main_create_compose_keyboards ()
     {
 	keyboard = gok_append_keyboard_from_file (keyboard, 
 						  gok_data_get_custom_compose_filename ());
-	custom_compose_kbd_name = keyboard->Name;
+	gok_args.custom_compose_kbd_name = keyboard->Name;
 	if (first == NULL)
 	    first = keyboard;
     }
@@ -1702,7 +1919,7 @@ gok_main_read_keyboards_from_dir (const char *directory, GokKeyboard *keyboard)
 * got from GConf.
 */
 static void
-gok_main_initialize_access_methods ()
+gok_main_initialize_access_methods (GokArgs *args)
 {
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
@@ -1715,7 +1932,7 @@ gok_main_initialize_access_methods ()
 					      &gconf_err);
     if (directory_name == NULL) {
 		gok_log_x ("Got NULL access method directory key from GConf");
-        gok_main_display_gconf_error ();
+		gok_main_display_gconf_error ();
 		exit (2);
     }
     else if (gconf_err != NULL) {
@@ -1725,7 +1942,10 @@ gok_main_initialize_access_methods ()
 			(_("could not access method directory key from GConf!"));
 		exit(2);		
     }
-    else if (!(gok_scanner_initialize(directory_name))) {
+    else if (!(gok_scanner_initialize(directory_name, 
+				      args->accessmethodname, 
+				      args->selectactionname, 
+				      args->scanactionname))) {
  		gok_main_display_error
 			(_("possibly unknown access method!"));
 		exit(2);
@@ -1748,31 +1968,111 @@ gok_main_initialize_wordcomplete ()
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
     gchar *directory_name = NULL;
+    gboolean user_copy = FALSE;
+    gchar *user_directory_name = NULL;
+    const gchar *homedir = g_get_home_dir ();
+
+    if (homedir) {
+	    user_directory_name = g_build_path (G_DIR_SEPARATOR_S, homedir, ".gnome2", "gok", NULL);
+    }
+    else 
+    {
+	    gok_log_x ("gok_main_initialize_wordcomplete: no home directory!");
+    }
 
     gconf_client = gconf_client_get_default ();
     
+    user_copy = gconf_client_get_bool (gconf_client,
+				       GOK_GCONF_PER_USER_DICTIONARY,
+				       &gconf_err);
+    if (gconf_err != NULL)
+    {
+	gok_log_x ("Error getting per-user dictionary key from GConf");
+	g_error_free (gconf_err);
+    }
+
     directory_name = gconf_client_get_string (gconf_client,
 					      GOK_GCONF_DICTIONARY_DIRECTORY,
 					      &gconf_err);
+
     if (directory_name == NULL)
     {
-	gok_log_x ("Got NULL dictionary directory key from GConf");
+	    gok_log_x ("Got NULL dictionary directory key from GConf");
     }
     else if (gconf_err != NULL)
     {
-	gok_log_x ("Error getting dictionary directory key from GConf");
-	g_error_free (gconf_err);
+	    gok_log_x ("Error getting dictionary directory key from GConf");
+	    g_error_free (gconf_err);
+	    return;
     }
-    else
+    if (user_copy)
     {
-	if ( !(gok_wordcomplete_open (gok_wordcomplete_get_default (),
-				      directory_name)) ||
-		! (gok_keyslotter_on (gok_wordcomplete_get_default (), KEYTYPE_WORDCOMPLETE))) {
-	    gok_log_x ("Error initializing word completion");
-	}
-	
-	g_free (directory_name);
+	    /* TODO: should we consider making the dict name at least a #define? */
+	    gchar *filename = g_build_filename (user_directory_name, "dictionary.txt", NULL);
+	    /* make sure dictionary exists; if not, copy from system. */
+	    if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+	    {
+		    if (!g_file_test (user_directory_name, G_FILE_TEST_EXISTS))
+		    {
+			    gok_log_x ("creating directory %s", user_directory_name);
+			    mkdir (user_directory_name, S_IRUSR | S_IWUSR | S_IXUSR);
+			    /* create user gok dir if necessary; */
+			    /*  we can discard the retval since we retest below  */
+		    }
+		    if (g_file_test (user_directory_name, G_FILE_TEST_IS_DIR))
+		    {
+			    gchar *content = NULL, *dict;
+			    GIOChannel *io;
+			    
+			    dict = g_build_filename (directory_name, "dictionary.txt", NULL);
+			    if (dict)
+			    {
+				    GError *error = NULL;
+				    gsize bytes;
+				    io = g_io_channel_new_file (filename, "w", &error);
+				    if (io && !error && (g_file_get_contents (dict, &content, NULL, &error)))
+				    {
+					g_io_channel_write_chars (io, content, -1, &bytes, &error);
+					g_free (content);
+					if (!error) 
+					{
+					    g_io_channel_shutdown (io, TRUE, &error);
+					    directory_name = user_directory_name;
+					}
+					else
+					{
+					    g_warning ("error writing user copy of gok system dictionary");
+					    g_free (error);
+					}
+				    }
+				    else
+				    {
+					g_warning ("Error creating user copy of gok system dictionary (%s)", filename);
+				    }
+			    }
+		    }
+		    else
+		    {
+			    g_warning ("Specified user dictionary path %s does not appear to be a directory", user_directory_name);
+		    }
+	    }
+	    else /* existed already */
+	    {
+		    gok_log ("Found file dictionary.txt in user dir %s", user_directory_name);
+		    directory_name = user_directory_name;
+	    }
+	    g_free (filename);
     }
+    
+    gok_log ("passing %s to gok-word-completion engine", directory_name);
+
+    if ( !(gok_wordcomplete_open (gok_wordcomplete_get_default (),
+				  directory_name)) ||
+	 ! (gok_keyslotter_on (gok_wordcomplete_get_default (), KEYTYPE_WORDCOMPLETE))) {
+	    gok_log_x ("Error initializing word completion");
+	    
+    }
+    g_free (directory_name);
 }
 
 /**
@@ -2296,7 +2596,33 @@ gok_main_display (void)
 gchar* 
 gok_main_get_access_method_override (void)
 {
-	return accessmethodname;
+	return gok_args.accessmethodname;
+}
+
+/**
+* gok_main_get_scan_override
+*
+* Call this to see if the "scan" action name was passed in from the command line
+*
+* returns: name of "scan" or "movehighlighter" action or NULL
+*/
+gchar* 
+gok_main_get_scan_override (void)
+{
+	return gok_args.scanactionname;
+}
+
+/**
+* gok_main_get_select_override
+*
+* Call this to see if the "select" action name was passed in from the command line
+*
+* returns: name of "select" or "outputselected" action or NULL
+*/
+gchar* 
+gok_main_get_select_override (void)
+{
+	return gok_args.selectactionname;
 }
 
 /**
@@ -2309,7 +2635,7 @@ gok_main_get_access_method_override (void)
 gboolean 
 gok_main_get_extras (void)
 {
-	return bExtras;
+	return gok_args.use_extras;
 }
 
 /**
@@ -2322,9 +2648,19 @@ gok_main_get_extras (void)
 gboolean 
 gok_main_get_login (void)
 {
-	return bLoginOption;
+	return gok_args.is_login;
 }
 
+/**
+* gok_main_get_inputdevice_name
+* 
+* returns: input device name
+*/
+gchar*
+gok_main_get_inputdevice_name ()
+{
+	return gok_args.inputdevicename;
+}
 
 /**
 * gok_main_has_xkb_extension
@@ -2381,7 +2717,7 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 						      XkbStickyKeysMask)) {
 #endif
 				xkb_ok = TRUE;
-				if (!bLoginOption) {
+				if (!gok_args.is_login) {
 				  dialog = gtk_message_dialog_new (NULL,
 								 0,
 								 GTK_MESSAGE_INFO,
@@ -2405,7 +2741,7 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 		XkbFreeKeyboard (xkb, 0, True);
 	}
 	if (xkb_ok == FALSE) {
-		if (!bLoginOption) {
+		if (!gok_args.is_login) {
                 /* post an error dialog */
 			dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -2461,7 +2797,7 @@ gok_main_help_cb (gpointer data)
 	GError *error = NULL;
 	gchar *helpuri = data;
 	/* TODO: detect error launching help, and give informative message */
-	gnome_help_display (helpuri, NULL, &error);
+	gnome_help_display_desktop (NULL, "gok", helpuri, NULL, &error);
 }
  
 void
