@@ -30,8 +30,25 @@ extern "C" {
 #include <libspi/Accessibility.h>
 #include <libspi/accessible.h>
 #include <libspi/application.h>
+#include <login-helper/Accessibility_LoginHelper.h>
+#include <login-helper/login-helper.h>
 #include "gok-keyboard.h"
 
+typedef struct _GokApplication GokApplication;
+
+/* GOK Bonobo type info and #defines */
+#define GOK_TYPE_APPLICATION        (gok_application_get_type ())
+#define GOK_APPLICATION(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), GOK_TYPE_APPLICATION, GokApplication))
+#define GOK_APPLICATION_CLASS(k)    (G_TYPE_CHECK_CLASS_CAST ((k), GOK_TYPE_APPLICATION, GokApplicationClass))
+
+struct _GokApplication {
+  LoginHelper     parent;
+};
+
+typedef struct _GokApplicationClass {
+  LoginHelperClass     parent_class;
+} GokApplicationClass;
+
 /* public */
 GtkWidget* gok_main_create_window (gboolean is_dock);
 void gok_main_set_cursor (GdkCursor *cursor);
@@ -60,9 +77,12 @@ void gok_main_display_error (gchar* ErrorString);
 void gok_main_display_gconf_error (void);
 gboolean gok_main_window_contains_pointer (void);
 Display *gok_main_display (void);
+gchar* gok_main_get_scan_override (void);
+gchar* gok_main_get_select_override (void);
 gchar* gok_main_get_access_method_override (void);
 gboolean gok_main_get_extras (void);
 gboolean gok_main_get_login (void);
+gchar* gok_main_get_inputdevice_name (void);
 void gok_main_check_accessibility (void);
 void gok_main_warn_if_corepointer_mode (gchar *message, gboolean always);
 
